@@ -149,6 +149,7 @@ typedef struct _PangoAttrIterator PangoAttrIterator;
  * @PANGO_ATTR_BACKGROUND_ALPHA: background alpha (#PangoAttrInt). Since 1.38
  * @PANGO_ATTR_ALLOW_BREAKS: whether breaks are allowed (#PangoAttrInt). Since 1.44
  * @PANGO_ATTR_SHOW: how to render invisible characters (#PangoAttrInt). Since 1.44
+ * @PANGO_ATTR_INSERT_HYPHENS: whether to insert hyphens at intra-word line breaks (#PangoAttrInt). Since 1.44
  *
  * The #PangoAttrType
  * distinguishes between different types of attributes. Along with the
@@ -187,6 +188,7 @@ typedef enum
   PANGO_ATTR_BACKGROUND_ALPHA,	/* PangoAttrInt */
   PANGO_ATTR_ALLOW_BREAKS,	/* PangoAttrInt */
   PANGO_ATTR_SHOW,		/* PangoAttrInt */
+  PANGO_ATTR_INSERT_HYPHENS,	/* PangoAttrInt */
 } PangoAttrType;
 
 /**
@@ -528,6 +530,8 @@ PANGO_AVAILABLE_IN_1_38
 PangoAttribute *pango_attr_background_alpha_new (guint16 alpha);
 PANGO_AVAILABLE_IN_1_44
 PangoAttribute *pango_attr_allow_breaks_new     (gboolean allow_breaks);
+PANGO_AVAILABLE_IN_1_44
+PangoAttribute *pango_attr_insert_hyphens_new   (gboolean insert_hyphens);
 
 /**
  * PangoShowFlags:
