@@ -26,6 +26,7 @@
 
 #include "pangocairo.h"
 #include "pangocairo-private.h"
+#include "pango-font-private.h"
 #include "pango-impl-utils.h"
 
 #define PANGO_CAIRO_FONT_PRIVATE(font)		\
@@ -184,179 +185,6 @@ _pango_cairo_font_install (PangoFont *font,
   return TRUE;
 }
 
-
-static int
-max_glyph_width (PangoLayout *layout)
-{
-  int max_width = 0;
-  GSList *l, *r;
-
-  for (l = pango_layout_get_lines_readonly (layout); l; l = l->next)
-    {
-      PangoLayoutLine *line = l->data;
-
-      for (r = line->runs; r; r = r->next)
-	{
-	  PangoGlyphString *glyphs = ((PangoGlyphItem *)r->data)->glyphs;
-	  int i;
-
-	  for (i = 0; i < glyphs->num_glyphs; i++)
-	    if (glyphs->glyphs[i].geometry.width > max_width)
-	      max_width = glyphs->glyphs[i].geometry.width;
-	}
-    }
-
-  return max_width;
-}
-
-typedef struct _PangoCairoFontMetricsInfo
-{
-  const char       *sample_str;
-  PangoFontMetrics *metrics;
-} PangoCairoFontMetricsInfo;
-
-PangoFontMetrics *
-_pango_cairo_font_get_metrics (PangoFont     *font,
-			       PangoLanguage *language)
-{
-  PangoCairoFont *cfont = (PangoCairoFont *) font;
-  PangoCairoFontPrivate *cf_priv = PANGO_CAIRO_FONT_PRIVATE (font);
-  PangoCairoFontMetricsInfo *info = NULL; /* Quiet gcc */
-  GSList *tmp_list;
-
-  const char *sample_str = pango_language_get_sample_string (language);
-
-  tmp_list = cf_priv->metrics_by_lang;
-  while (tmp_list)
-    {
-      info = tmp_list->data;
-
-      if (info->sample_str == sample_str)    /* We _don't_ need strcmp */
-	break;
-
-      tmp_list = tmp_list->next;
-    }
-
-  if (!tmp_list)
-    {
-      PangoFontMap *fontmap;
-      PangoContext *context;
-      cairo_font_options_t *font_options;
-      PangoLayout *layout;
-      PangoRectangle extents;
-      PangoFontDescription *desc;
-      cairo_scaled_font_t *scaled_font;
-      cairo_matrix_t cairo_matrix;
-      PangoMatrix pango_matrix;
-      PangoMatrix identity = PANGO_MATRIX_INIT;
-      glong sample_str_width;
-
-      int height, shift;
-
-      /* XXX this is racy.  need a ref'ing getter... */
-      fontmap = pango_font_get_font_map (font);
-      if (!fontmap)
-        return pango_font_metrics_new ();
-      fontmap = g_object_ref (fontmap);
-
-      info = g_slice_new0 (PangoCairoFontMetricsInfo);
-
-      cf_priv->metrics_by_lang = g_slist_prepend (cf_priv->metrics_by_lang, info);
-
-      info->sample_str = sample_str;
-
-      scaled_font = _pango_cairo_font_private_get_scaled_font (cf_priv);
-
-      context = pango_font_map_create_context (fontmap);
-      pango_context_set_language (context, language);
-
-      font_options = cairo_font_options_create ();
-      cairo_scaled_font_get_font_options (scaled_font, font_options);
-      pango_cairo_context_set_font_options (context, font_options);
-      cairo_font_options_destroy (font_options);
-
-      info->metrics = (* PANGO_CAIRO_FONT_GET_IFACE (font)->create_base_metrics_for_context) (cfont, context);
-
-      /* We now need to adjust the base metrics for ctm */
-      cairo_scaled_font_get_ctm (scaled_font, &cairo_matrix);
-      pango_matrix.xx = cairo_matrix.xx;
-      pango_matrix.yx = cairo_matrix.yx;
-      pango_matrix.xy = cairo_matrix.xy;
-      pango_matrix.yy = cairo_matrix.yy;
-      pango_matrix.x0 = 0;
-      pango_matrix.y0 = 0;
-      if (G_UNLIKELY (0 != memcmp (&identity, &pango_matrix, 4 * sizeof (double))))
-        {
-	  double xscale = pango_matrix_get_font_scale_factor (&pango_matrix);
-	  if (xscale) xscale = 1 / xscale;
-
-	  info->metrics->ascent *= xscale;
-	  info->metrics->descent *= xscale;
-	  info->metrics->underline_position *= xscale;
-	  info->metrics->underline_thickness *= xscale;
-	  info->metrics->strikethrough_position *= xscale;
-	  info->metrics->strikethrough_thickness *= xscale;
-	}
-
-
-      /* Set the matrix on the context so we don't have to adjust the derived
-       * metrics. */
-      pango_context_set_matrix (context, &pango_matrix);
-
-      /* Update approximate_*_width now */
-      layout = pango_layout_new (context);
-      desc = pango_font_describe_with_absolute_size (font);
-      pango_layout_set_font_description (layout, desc);
-      pango_font_description_free (desc);
-
-      pango_layout_set_text (layout, sample_str, -1);
-      pango_layout_get_extents (layout, NULL, &extents);
-
-      sample_str_width = pango_utf8_strwidth (sample_str);
-      g_assert (sample_str_width > 0);
-      info->metrics->approximate_char_width = extents.width / sample_str_width;
-
-      pango_layout_set_text (layout, "0123456789", -1);
-      info->metrics->approximate_digit_width = max_glyph_width (layout);
-
-      g_object_unref (layout);
-
-
-      /* We may actually reuse ascent/descent we got from cairo here.  that's
-       * in cf_priv->font_extents.
-       */
-      height = info->metrics->ascent + info->metrics->descent;
-      switch (cf_priv->gravity)
-	{
-	  default:
-	  case PANGO_GRAVITY_AUTO:
-	  case PANGO_GRAVITY_SOUTH:
-	    break;
-	  case PANGO_GRAVITY_NORTH:
-	    info->metrics->ascent = info->metrics->descent;
-	    break;
-	  case PANGO_GRAVITY_EAST:
-	  case PANGO_GRAVITY_WEST:
-	    {
-	      int ascent = height / 2;
-	      if (cf_priv->is_hinted)
-	        ascent = PANGO_UNITS_ROUND (ascent);
-	      info->metrics->ascent = ascent;
-	    }
-	}
-      shift = (height - info->metrics->ascent) - info->metrics->descent;
-      info->metrics->descent += shift;
-      info->metrics->underline_position -= shift;
-      info->metrics->strikethrough_position -= shift;
-      info->metrics->ascent = height - info->metrics->descent;
-
-      g_object_unref (context);
-      g_object_unref (fontmap);
-    }
-
-  return pango_font_metrics_ref (info->metrics);
-}
-
 static PangoCairoFontHexBoxInfo *
 _pango_cairo_font_private_get_hex_box_info (PangoCairoFontPrivate *cf_priv)
 {
@@ -624,14 +452,6 @@ _pango_cairo_font_private_initialize (PangoCairoFontPrivate      *cf_priv,
   cf_priv->scaled_font = NULL;
   cf_priv->hbi = NULL;
   cf_priv->glyph_extents_cache = NULL;
-  cf_priv->metrics_by_lang = NULL;
-}
-
-static void
-free_metrics_info (PangoCairoFontMetricsInfo *info)
-{
-  pango_font_metrics_unref (info->metrics);
-  g_slice_free (PangoCairoFontMetricsInfo, info);
 }
 
 void
@@ -649,10 +469,6 @@ _pango_cairo_font_private_finalize (PangoCairoFontPrivate *cf_priv)
   if (cf_priv->glyph_extents_cache)
     g_free (cf_priv->glyph_extents_cache);
   cf_priv->glyph_extents_cache = NULL;
-
-  g_slist_foreach (cf_priv->metrics_by_lang, (GFunc)free_metrics_info, NULL);
-  g_slist_free (cf_priv->metrics_by_lang);
-  cf_priv->metrics_by_lang = NULL;
 }
 
 gboolean
@@ -661,6 +477,45 @@ _pango_cairo_font_private_is_metrics_hinted (PangoCairoFontPrivate *cf_priv)
   return cf_priv->is_hinted;
 }
 
+static void
+get_space_extents (PangoCairoFontPrivate *cf_priv,
+                   PangoRectangle        *ink_rect,
+                   PangoRectangle        *logical_rect)
+{
+  const char hexdigits[] = "0123456789ABCDEF";
+  char c[2] = {0, 0};
+  int i;
+  double hex_width;
+  int width;
+
+  /* we don't render missing spaces as hex boxes,
+   * so come up with some width to use. For lack
+   * of anything better, use average hex digit width.
+   */
+
+  hex_width = 0;
+  for (i = 0 ; i < 16 ; i++)
+    {
+      cairo_text_extents_t extents;
+
+      c[0] = hexdigits[i];
+      cairo_scaled_font_text_extents (cf_priv->scaled_font, c, &extents);
+      hex_width += extents.width;
+    }
+  width = pango_units_from_double (hex_width / 16);
+
+  if (ink_rect)
+    {
+      ink_rect->x = ink_rect->y = ink_rect->height = 0;
+      ink_rect->width = width;
+    }
+  if (logical_rect)
+    {
+      *logical_rect = cf_priv->font_extents;
+      logical_rect->width = width;
+    }
+}
+
 static void
 _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_priv,
 						     PangoGlyph             glyph,
@@ -671,6 +526,14 @@ _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_p
   gunichar ch;
   gint rows, cols;
 
+  ch = glyph & ~PANGO_GLYPH_UNKNOWN_FLAG;
+
+  if (ch == 0x20 || ch == 0x2423)
+    {
+      get_space_extents (cf_priv, ink_rect, logical_rect);
+      return;
+    }
+
   hbi = _pango_cairo_font_private_get_hex_box_info (cf_priv);
   if (!hbi)
     {
@@ -678,13 +541,20 @@ _pango_cairo_font_private_get_glyph_extents_missing (PangoCairoFontPrivate *cf_p
       return;
     }
 
-  ch = glyph & ~PANGO_GLYPH_UNKNOWN_FLAG;
-
-  rows = hbi->rows;
   if (G_UNLIKELY (glyph == PANGO_GLYPH_INVALID_INPUT || ch > 0x10FFFF))
-    cols = 1;
+    {
+      rows = hbi->rows;
+      cols = 1;
+    }
+  else if (pango_get_ignorable_size (ch, &rows, &cols))
+    {
+      /* We special-case ignorables when rendering hex boxes */
+    }
   else
-    cols = ((glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 6 : 4) / rows;
+    {
+      rows = hbi->rows;
+      cols = ((glyph & ~PANGO_GLYPH_UNKNOWN_FLAG) > 0xffff ? 6 : 4) / rows;
+    }
 
   if (ink_rect)
     {
@@ -758,6 +628,7 @@ _pango_cairo_font_private_glyph_extents_cache_init (PangoCairoFontPrivate *cf_pr
   return TRUE;
 }
 
+
 /* Fills in the glyph extents cache entry
  */
 static void
